from qgis.PyQt.QtCore import (QCoreApplication, QEvent, QFileInfo, QItemSelectionModel, QObject, QSettings, QSize, QSizeF, Qt, QTimer, pyqtSignal)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QButtonGroup, QCheckBox, QComboBox, QDialog, QDialogButtonBox, QFileDialog,
                                 QGraphicsScene, QGraphicsView, QGroupBox, QInputDialog, QLabel, QListView, QMdiSubWindow, QMenu, QPushButton, QSizePolicy,
                                 QScrollArea, QSlider, QTabWidget, QTableView, QToolBar, QToolTip, QVBoxLayout, QTreeWidget, QTreeWidgetItem, QWidget)

from qgis.PyQt.QtGui import (QColor, QCursor, QIcon, QImage, QKeySequence, QPixmap, QStandardItemModel, QTextDocument)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)

from qgis.core import (QgsApplication, Qgis, QgsCoordinateReferenceSystem, QgsCoordinateTransform, QgsGeometry, QgsLayerTree, QgsLayerTreeLayer,
                       QgsLayerTreeGroup,  QgsLayerTreeModel, QgsPointXY, QgsProject, QgsRasterLayer, QgsRectangle, QgsSpatialIndex, QgsStyle, QgsVectorLayer)

from qgis.core import (QgsLayerTreeNode, QgsMapLayerProxyModel, QgsUnitTypes)
from qgis.gui import (QgsCollapsibleGroupBox, QgsColorWheel, QgsMapLayerComboBox, QgsLayerTreeMapCanvasBridge, QgsLayerTreeView,
                      QgsMapCanvas, QgsMapTool, QgsMapToolPan,  QgsMapToolZoom, QgsMessageBar, QgsRasterTransparencyWidget,
                      QgsRendererPropertiesDialog, QgsRendererRasterPropertiesWidget, QgsRubberBand )


from os.path import (basename, dirname, exists, isfile, join, splitext)
from autoLoaderRaster.utils import (changeButtonIcon, cleanPath, convertSize, fixeLabelsFileDialog, fixeMaxDim, funnyPushButton,
                                    getComboLayer, getComboLayerIndex, getListLayers, getThemeIcon, GroupComboBox,
                                    iff, myPushButton, printerDlg, saveDlg, textForMenu, windowIcon)

from datetime import datetime as thedatetime
from math import ceil


class legendTreeWidget( QTreeWidget ):
    def __init__(self, settings, parent=None):
        QTreeWidget.__init__(self, parent)

        self.vectors, self.rasters, self.alrRasters = None, None, None
        self.parent = parent
        self.ihmWindow = self.parent.parent().parent()
        
        self.setItemsExpandable(True)
        self.setAnimated(True)
        self.setDragEnabled(True)
        self.setDropIndicatorShown(True)
        self.setDragDropMode(QAbstractItemView.InternalMove)
        self.header().setDefaultSectionSize(150)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setMinimumWidth(300)
        self.setGeometry(500, 200, 300, 400)
        self.settings = settings

        root=self.invisibleRootItem()
        root.setData(0, Qt.ToolTipRole,"root")
        self.setRootIsDecorated( False )
        self.setHeaderLabels([QApplication.translate("autoLoaderRasterPlugin", "Groups and Layers"), QApplication.translate("autoLoaderRasterPlugin", "Opacity")])

        styleSheet = "QTreeWidget { border: 1px solid;  border-radius: 5px; opacity: 0.5; background-color: rgba(255, 255, 255, 50);}"
        styleSheet+= "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }"

        self.imgChecked = getThemeIcon("checked.png").replace("\\","/")
        self.imgUnChecked = getThemeIcon("unchecked.png").replace("\\","/")

        styleSheet+= "QTreeWidget {opacity: 0.5;  background-color: rgba(255, 255, 255, 210);}"
        styleSheet+= "QTreeWidget::indicator:checked {image: url(%s);}" % self.imgChecked
        styleSheet+= "QTreeWidget::indicator:unchecked  {image: url(%s);}" % self.imgUnChecked


        self.setStyleSheet(styleSheet)
        
        self.setColumnWidth(0, 200)
        self.setColumnWidth(1, 80)
        
        
        self.setColumnCount(2)
        
        self.styleSlider = """
        QSlider::groove:horizontal {border: 1px solid #bbb; background: white; height: 10px; border-radius: 4px;}
        QSlider::sub-page:horizontal {background: qlineargradient(x1: 0, y1: 0,    x2: 0, y2: 1,  stop: 0 #66e, stop: 1 #bbf); background: qlineargradient(x1: 0, y1: 0.2, x2: 1, y2: 1,  stop: 0 #bbf, stop: 1 #55f);border: 1px solid #777;height: 10px;border-radius: 4px;}
        QSlider::add-page:horizontal {background: #fff; border: 1px solid #777; height: 10px; border-radius: 4px;}
        QSlider::handle:horizontal {background: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 #eee, stop:1 #ccc); border: 1px solid #777; width: 13px; margin-top: -2px; margin-bottom: -2px; border-radius: 4px;}
        QSlider::handle:horizontal:hover {background: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 #fff, stop:1 #ddd); border: 1px solid #444; border-radius: 4px;}
        QSlider::sub-page:horizontal:disabled {background: #bbb; border-color: #999;}
        QSlider::add-page:horizontal:disabled {background: #eee; border-color: #999;}
        QSlider::handle:horizontal:disabled {background: #eee; border: 1px solid #aaa; border-radius: 4px;}
        """

        self.itemChanged.connect(self.treeWidgetItemChanged)

        self.parent.installEventFilter(self)

    def eventFilter(self, obj, event):
        if obj is self.parent:
           if event.type() == event.Resize: self.resizeObjects()
        return QObject.eventFilter(self, obj, event)
           
    def expandTreeWidget(self):
        if self.sender().objectName() == "buttonExpandAllNodes" : self.expandAll()
        else : self.collapseAll()

    def resizeObjects(self):
        self.setGeometry(10, 30, self.parent.width()-30, self.parent.height()-40) 
        self.setColumnWidth(0, self.parent.width()-150)
        self.setColumnWidth(1, 80)
        self.ihmWindow.buttonExpandAllNodes.setGeometry(self.parent.width()-60, 5, 15, 15)
        self.ihmWindow.buttonReduceAllNodes.setGeometry(self.parent.width()-40, 5, 15, 15)
        #NEW 3.2.0
        self.ihmWindow.closeLegend.setGeometry(self.parent.width()-20, 5, 15, 15)

    def getGroupParent(self, nameGroup): 
        results = self.findItems(nameGroup, Qt.MatchContains|Qt.MatchRecursive, 0)
        for item in results :
            return item
        return None

    def removeAllLayers(self):
        root = self.invisibleRootItem()
        for i in range(root.childCount()):
            group = root.child(i)
            if group :
               for j in range(group.childCount()-1, -1, -1,) : group.removeChild(group.child(j))

    def setMapLayers(self):
        layersMapA, layersMapB = [], []
        root = self.invisibleRootItem()
        for i in range(root.childCount()-1, -1, -1):
            group = root.child(i)
            if group :
                for j in range(group.childCount()-1, -1, -1):
                    item = group.child(j)
                    if item :
                        layer = self.layerItem(item.whatsThis(0))
                        if layer :
                           if group.text(0) == "%s A" % self.ihmWindow.txtMap : layersMapA.insert(0, layer)
                           elif group.text(0) == "%s B" % self.ihmWindow.txtMap : layersMapB.insert(0, layer)
                           else :
                                layersMapA.insert(0, layer)
                                layersMapB.insert(0, layer)

        #NEW 3.2.0
        if layersMapA != [] :
            self.ihmWindow.refCrs = layersMapA[0].crs()
            self.ihmWindow.mapViewerLayer.setDestinationCrs(self.ihmWindow.refCrs)
        if layersMapB != [] :
            refCrsB = layersMapB[0].crs()
            self.ihmWindow.mapViewerLayerDouble.setDestinationCrs(refCrsB)
            
        self.ihmWindow.mapViewerLayer.setLayers(layersMapA)
        self.ihmWindow.mapViewerLayerDouble.setLayers(layersMapB)

    def layerItem(self, whatsThis):
        try : infos = whatsThis.split("|")
        except : return None

        findLayer = None
      
        layerId = infos[1]
        root = self.ihmWindow.project.layerTreeRoot()
        allLayers = root.layerOrder()
        for layer in allLayers:
            mapLayer = root.findLayer(layerId)
            if mapLayer : return mapLayer.layer()

        if not findLayer :
            root = QgsProject.instance().layerTreeRoot()
            allLayers = root.layerOrder()
            for layer in allLayers:
                mapLayer = root.findLayer(layerId)
                if mapLayer : return mapLayer.layer()
        
        return findLayer        


    def updateValueSlider(self, value):
        layer = self.layerItem(self.sender().toolTip())
        if layer :
           opacity = 1.00-(value/100.00)            
           layer.renderer().setOpacity(opacity) if type(layer) == QgsRasterLayer else layer.setOpacity(opacity)
           self.ihmWindow.mapViewerLayer.refresh()
           self.ihmWindow.mapViewerLayerDouble.refresh()

        item = self.currentItem()
        if not item : return
        item.setData(1, Qt.ToolTipRole, value)

    def treeWidgetItemChanged(self, item, column):
        if item : 
            infos = item.whatsThis(0)
            if infos == "" : return
            infos = infos.split("|")
            if not infos[0] in ("groups", "alrgroup") : self.setItemVisibility(item) 
            else :
               for i in range(item.childCount()): self.setItemVisibility(item.child(i))

                
    def setItemVisibility(self, item):
        layer = self.layerItem(item.whatsThis(0))
        if layer :
            visibility = (item.checkState(0) == Qt.Checked)
            root = self.ihmWindow.project.layerTreeRoot()
            mapLayer = root.findLayer(layer.id())
            if mapLayer :
               mapLayer.setItemVisibilityChecked(visibility)
               self.updateVisibilityInB(self.ihmWindow.mapViewerLayer, item, layer, visibility)
               self.updateVisibilityInB(self.ihmWindow.mapViewerLayerDouble, item, layer, visibility)
               
    def updateVisibilityInB(self, mapViewer, item, layer, visibility):
        for maplayer in mapViewer.layers () :
            if maplayer.id() == layer.id() :
               slider = self.itemWidget(item, 1)
               opacity = 1 if not slider else 1.00-(slider.value()/100.00)
               #NEW 3.2.0
               try :
                   maplayer.setOpacity(opacity) if visibility else maplayer.setOpacity(0)
                   mapViewer.refresh()
               except : pass    
 
    def mouseDoubleClickEvent(self, event): 
        item = self.currentItem()
        if not item : return
        layer = self.layerItem(item.whatsThis(0))        
        if layer :
           self.ihmWindow.setPropertiesLayer(layer, item)

           if type(layer) == QgsVectorLayer :
              renderer = layer.renderer() 
              icon = None              
              for symbolItem in renderer.legendSymbolItems() :
                  image = symbolItem.symbol().asImage(QSize(32, 32))
                  pix = QPixmap.fromImage(image)
                  icon = QIcon()
                  icon.addPixmap(pix)
                  break
              if icon : item.setIcon(0, icon)   


    def dragMoveEvent(self, event):
        if self.currentItem().childCount() > 0 : return
        
        role = Qt.ToolTipRole
        itemToDropIn = self.itemAt(event.pos())
        itemBeingDragged = self.currentItem()
        okList = self.settings[itemBeingDragged.data(0,role)][0]

        if itemToDropIn is None: itemToDropIn = self.invisibleRootItem()

        if itemToDropIn.data(0, role) in okList:
            super(legendTreeWidget, self).dragMoveEvent(event)            
            return
        else:
            parent = itemToDropIn.parent()
            if parent is None : parent=self.invisibleRootItem()
            if parent.data(0,role) in okList:
                super(legendTreeWidget, self).dragMoveEvent(event)
                return
        event.ignore()

    def dropEvent(self, event):
        if self.currentItem().childCount() > 0 : return
        
        role = Qt.ToolTipRole 

        itemBeingDragged = self.currentItem()
        valueSlider = itemBeingDragged.data(1, role)
        okList = self.settings[itemBeingDragged.data(0, role)][0]
 
        oldParent = itemBeingDragged.parent()
        if oldParent is None: oldParent = self.invisibleRootItem()
        oldIndex = oldParent.indexOfChild(itemBeingDragged)

        super(legendTreeWidget,self).dropEvent(event)

        newParent = itemBeingDragged.parent()
        if newParent is None : newParent = self.invisibleRootItem()

        if not newParent.data(0, role) in okList :
            newParent.removeChild(itemBeingDragged)
            oldParent.insertChild(oldIndex, itemBeingDragged)
            
        self.setSlider(itemBeingDragged, valueSlider)
        self.setMapLayers() 
            
    def setSlider(self, item, value):
        slider = QSlider(self)
        slider.setMinimumSize(100, 20)
        slider.setMaximumSize(100, 20)
        slider.setGeometry(10, 100, 100, 20)
        slider.setOrientation(Qt.Horizontal)
        slider.setMinimum(0)
        slider.setMaximum(100)
        slider.setValue(value)
        slider.setPageStep(10)
        slider.setSingleStep(1)
        slider.setInvertedAppearance(True)
        slider.setStyleSheet(self.styleSlider) 
        self.setItemWidget(item, 1, slider)
        item.setData(1, Qt.ToolTipRole, slider.value())
        slider.setToolTip(item.whatsThis(0))
            
        slider.valueChanged.connect(self.updateValueSlider)


    def setDataItem(self, groupText, findText, itemText, layerId, itemIcon, groupKey, category, group):
        #SUBSTITUTE ITEM IF EXISTS
        group = self.getGroupParent(groupText)
        item = None
        #RE-CREATE GROUP IF NO GROUP ?
        if group :
           for i in range(group.childCount()):
               if group.child(i).text(0).startswith(findText) :
                  group.child(i).setData(0, Qt.DisplayRole, itemText)
                  group.child(i).setWhatsThis(0, "%s|%s" % (category, layerId))
                  item = group.child(i)

                  if category.find("group") == -1 : self.setSlider(group.child(i), 0)
                  
                  break

           if not item :
              #BUG TO FINISH ... BUG WITH DUPLICATE LAYER 
              try :
                 layer = self.addItem([itemText], groupKey, layerId, itemIcon, category, group)
                 layer.setCheckState(0, Qt.Checked)
              except : pass               

    def addItem(self, strings, category, fileId, iconlayer, parentname=None, parent=None):
        if category not in self.settings:  return False
        if parent is None : parent = self.invisibleRootItem()
        
        item = QTreeWidgetItem(parent, strings)
        item.setData(0, Qt.ToolTipRole, category)
        item.setWhatsThis(0, "%s|%s" % (category, fileId)) if parent else item.setWhatsThis(0, "%s|%s" % (category, parentname))
        item.setExpanded(True)
        item.setFlags(self.settings[category][1])
        if type(iconlayer) == str : iconlayer = QIcon(iconlayer)
        item.setIcon(0, iconlayer)
        
        if category.find("group") == -1 : self.setSlider(item, 0)
    
        return item

    def removeItemWithId(self, groupText, layerId):
        group = self.getGroupParent(groupText)
        if group :
           for i in range(group.childCount()):
               item = group.child(i)
               infos = item.whatsThis(0).split("|")
               if layerId == infos[1] :
                  group.removeChild(item)
                  break



#---------------------------------------------------------------------------
# (c) Christophe MASSE 2013 - 2024
# revision code : 11-12-2020
# CLASS uiDoubleFocusGlassesPlugin - AUTOLOADERRASTER PLUGIN
#---------------------------------------------------------------------------
class uiDoubleFocusGlassesPlugin(QDialog):
    
    def __init__(self, parent):
        super(uiDoubleFocusGlassesPlugin, self).__init__()
        self.setObjectName("uiDoubleFocusGlassesPlugin")
        self.parent = parent
        self.styleSheetPushButton = self.parent.styleSheetPushButton
        self.styleComboBox = self.parent.styleComboBox
        self.resize(1000, 680)
        self.setWindowFlags(Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
        self.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin", "Double glass viewer"))
        windowIcon(self, "checkdoubleglasses.png")

        self.iface = self.parent.iface
        self.saveFolder = join(dirname(__file__),"works")
        self.layers, self.currentLayers = [], []

        #NEW 3.2.0
        self.transformer = QgsCoordinateTransform()

       
        self.canvas = self.parent.canvas
        self.rasterIcon = QIcon(getThemeIcon("raster.png"))
        self.vectorIcon = QIcon(getThemeIcon("vector.png"))
        self.groupIcon = QIcon(getThemeIcon("group.png"))
        self.imgChecked = getThemeIcon("checked.png").replace("\\","/")
        self.imgUnChecked = getThemeIcon("unchecked.png").replace("\\","/")        

        self.printMapview = QTextDocument()

        self.tableListLayers, self.comboLayersCurrentLayerId = {}, 0

        self.model = QStandardItemModel()
        self.view = QTableView()
        self.view.setModel( self.model )
        self.view.horizontalHeader().setVisible(False)
        self.view.verticalHeader().setVisible(False)
        self.view.setColumnHidden(1, True)
        self.view.verticalHeader().setDefaultSectionSize(20)
        self.view.setStyleSheet("QTableView {min-width: 800px; max-width: 1500px; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}")
        self.view.setShowGrid(False)

        #NEW 3.2.0
        self.listLayers = GroupComboBox(self)
        self.listLayers.setModel(self.model)
        self.listLayers.setView(self.view)
        self.listLayers.setObjectName("listLayers")
        self.listLayers.setStyleSheet(self.styleComboBox)


        self.buttonToolBarNav = funnyPushButton(self) 
        self.buttonToolBarNav.initPushButton(25, 25, 40, 90, "buttonToolBarNav", "", QApplication.translate("autoLoaderRasterPlugin", "Show/Hide navigation toolbar"), True, getThemeIcon("navigation_on.png"), 25, 25, True)
        self.buttonToolBarNav.setShortcut(QKeySequence("Ctrl+N"))
        
        self.buttonToolBarLegend = funnyPushButton(self) 
        self.buttonToolBarLegend.initPushButton(25, 25, 40, 90, "buttonToolBarLegend", "", QApplication.translate("autoLoaderRasterPlugin", "Show/Hide legend toolbar"), True, getThemeIcon("legend_off.png"), 25, 25, True)
        self.buttonToolBarLegend.setShortcut(QKeySequence("Ctrl+L"))

        self.initDir = join(dirname(__file__), "works")
        
        self.showGrids = False
        self.buttonToolBarGrid = funnyPushButton(self) 
        self.buttonToolBarGrid.initPushButton(25, 25, 40, 90, "buttonToolBarGrid", "", QApplication.translate("autoLoaderRasterPlugin", "Show/Hide Grid Maps"), True, getThemeIcon("grid_off.png"), 25, 25, True)
        self.buttonToolBarGrid.setShortcut(QKeySequence("Ctrl+G"))

        toolTip = QApplication.translate("autoLoaderRasterPlugin","Actions ...")
        self.buttonActions = funnyPushButton(self) 
        self.buttonActions.initPushButton(40, 25, 40, 90, "buttonActions", "", toolTip, True, getThemeIcon("actions.png"), 40, 25, True)
 
        self.buttonReverse = funnyPushButton(self) 
        self.buttonReverse.initPushButton(25, 25, 40, 90, "buttonReverse", "", QApplication.translate("autoLoaderRasterPlugin", "Swap members"), True, getThemeIcon("reverse.png"), 25, 25, False)        
        self.buttonReverse.setShortcut(QKeySequence("Ctrl+R"))
        self.buttonReverse.clicked.connect(self.swapListMember)

        self.buttonToolBarNav.clicked.connect(self.getToolBarNav)
        self.buttonToolBarLegend.clicked.connect(self.getToolBarLegend)
        self.buttonToolBarGrid.clicked.connect(self.getGridMaps)


        self.mapViewerLayer = QgsMapCanvas(self)
        self.refCrs = self.parent.projectAlr.crs()
        #NEW 3.2.0
        self.barInfo = QgsMessageBar(self.mapViewerLayer)
        self.barInfo.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding) #QSizePolicy.Minimum, QSizePolicy.Fixed)

        self.labelScale = QLabel(self)
        self.labelScale.setStyleSheet("QLabel { font-size: 11px; border: 1px solid white; border-radius: 5px;  background-color: rgba(255, 255, 255, 50);}")
        self.labelScale.setAlignment(Qt.AlignCenter)
        
        self.widgetmapViewerLayerGridParams = actionsWindows(self.mapViewerLayer, "widgetmapViewerLayerGridParams", "%s ...:" % QApplication.translate("autoLoaderRasterPlugin","Params"), 130, 200, False)


        #NEW 3.2.0
        self.closeGridParams = funnyPushButton(self.widgetmapViewerLayerGridParams) 
        self.closeGridParams.initPushButton(15, 15, 110, 5, "closeGridParams", "", QApplication.translate("autoLoaderRasterPlugin", "Close"), True, getThemeIcon("closenb.png"), 15, 15, True)
        self.closeGridParams.clicked.connect(self.getGridParams)

        self.lblListParamsGrid = QLabel(self.widgetmapViewerLayerGridParams)
        self.lblListParamsGrid.setObjectName("lblListParamsGrid")
        self.lblListParamsGrid.setText("%s :" % QApplication.translate("autoLoaderRasterPlugin","Grid"))
        self.lblListParamsGrid.setAlignment(Qt.AlignRight)
        self.lblListParamsGrid.setGeometry(10, 35, 50, 25)


        styleComboBox = """
                          QComboBox { border-style: outset; border-radius: 3px;  selection-color: black; selection-background-color: lightgray;}
                          QComboBox:editable {  background-color: white; }
                          QComboBox:!editable, QComboBox::drop-down:editable { font-color: black; color : black; border-left-width: 1px; border-left-color: black; border-left-style: solid; border-top-right-radius: 3px; border-right-color: black;}
                          QComboBox:!editable:on, QComboBox::drop-down:editable:on {color : black; font-color: black;  }
                          QComboBox:on { padding-top: 3px; padding-left: 4px; }
                          QComboBox::drop-down {color: black; subcontrol-origin: padding; subcontrol-position: top right; width: 15px; border-left-width: 1px; border-left-color: darkgray; border-left-style: solid; border-top-right-radius: 3px; border-bottom-right-radius: 3px;}
                          QComboBox::down-arrow { image: url(%s); }
                          QComboBox::down-arrow:on {top: 1px; left: 1px; }
                          QComboBox QAbstractItemView { border: 2px solid darkgray;  selection-background-color: lightgray; color: black; font-color: black;}
                        """ % getThemeIcon("nodes_expand.png").replace("\\","/") 


        self.listParamsGrid = QComboBox(self.widgetmapViewerLayerGridParams)
        self.listParamsGrid.setObjectName("listParamsGrid")
        self.listParamsGrid.setStyleSheet(styleComboBox)
        self.listParamsGrid.addItems([ "2", "4", "5", "10"])
        self.listParamsGrid.setCurrentIndex(1)
        self.listParamsGrid.setGeometry(65, 30, 50, 25)
        self.listParamsGrid.currentIndexChanged.connect(self.updateGridMaps)

        self.lblMireColor = QLabel(self.widgetmapViewerLayerGridParams)
        self.lblMireColor.setObjectName("lblMireColor")
        self.lblMireColor.setText("%s :" % QApplication.translate("autoLoaderRasterPlugin","Colors"))
        self.lblMireColor.setAlignment(Qt.AlignCenter)
        self.lblMireColor.setGeometry(10, 60, 110, 25)


        self.mireColor = QgsColorWheel(self.widgetmapViewerLayerGridParams) 
        self.mireColor.setObjectName("mireColor")
        self.mireColor.setStyleSheet("{ border: 1px solid; border-radius: 5px;  opacity: 0.5; background-color: rgba(255, 255, 255, 50);}")
        self.mireColor.setGeometry(10, 85, 110, 110)
        self.mireColor.setBaseSize(24,24)
        self.mireColor.setToolTip(QApplication.translate("autoLoaderRasterPlugin", "Color used for the extent"))
        self.mireColor.setColor(Qt.blue)
        self.mireColor.colorChanged.connect(self.setColors)

        self.widgetmapViewerLayerGridParams.setVisible(False)

        self.contextMnuActions(self.buttonActions)


        self.rbDynaMire = QgsRubberBand(self.mapViewerLayer, True)
        self.rbDynaMire.setColor(Qt.blue)
        self.rbDynaMire.setWidth(1)
        self.rbDynaMire.setLineStyle(Qt.DashDotLine) 
        
        self.rb = QgsRubberBand(self.mapViewerLayer, True)
        self.rbaxesh = QgsRubberBand(self.mapViewerLayer, True)
        self.rbaxesv = QgsRubberBand(self.mapViewerLayer, True)
        self.rb.setColor(Qt.blue)
        self.rb.setWidth(2)
        self.rbaxesh.setColor(Qt.blue)
        self.rbaxesv.setColor(Qt.blue)

        self.legende = actionsWindows(self.mapViewerLayer, "widgetConsole", "%s ..." % QApplication.translate("autoLoaderRasterPlugin", "Legende"), 340, 450, True )

        self.widgetMapView = actionsWindows(self.mapViewerLayer, "widgetConsole", "%s ..." % QApplication.translate("autoLoaderRasterPlugin", "Actions"), 340, 65, False)
        ctrlMap(self, self.widgetMapView, True, False, False, 5, 0, 350, 30, False)
        
        self.widgetMapView.setGeometry(0, 0, 340, 65)
        self.widgetmapViewerLayerGridParams.setGeometry(341, 0, 120, 150) 
        self.legende.setGeometry(0, 65, 340, 450)

        self.toolPan = QgsMapToolPan(self.mapViewerLayer)
        self.toolZoomIn = QgsMapToolZoom(self.mapViewerLayer, False) 
        self.toolZoomOut = QgsMapToolZoom(self.mapViewerLayer, True)

        self.buttonCatchExtent.clicked.connect(self.catchExtent)
        self.buttonPan.clicked.connect(self.pan)
        self.buttonZoomIn.clicked.connect(self.zoomIn)
        self.buttonZoomOut.clicked.connect(self.zoomOut)

        self.contextMnuZoomAll(self.buttonZoomAll)
        self.buttonZoomPrevious.clicked.connect(self.zoomPrevious)
        self.buttonZoomNext.clicked.connect(self.zoomNext)
        self.buttonSendExtent.clicked.connect(self.sendExtentToMapCanvas)

        #NEW 3.2.0 
        self.model2 = QStandardItemModel()
        self.view2 = QTableView()
        self.view2.setModel( self.model2 )
        self.view2.horizontalHeader().setVisible(False)
        self.view2.verticalHeader().setVisible(False)
        self.view2.setColumnHidden(1, True)
        self.view2.verticalHeader().setDefaultSectionSize(20)
        self.view2.setStyleSheet("QTableView {min-width: 800px; max-width: 1500px; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}")
        self.view2.setShowGrid(False)
        
        self.listLayersDouble = GroupComboBox(self) #QComboBox(self)
        self.listLayersDouble.setStyleSheet(self.styleComboBox)        
        self.listLayersDouble.setModel(self.model2)
        self.listLayersDouble.setView(self.view2)         
        self.listLayersDouble.setObjectName("listLayersDouble")
       
        self.mapViewerLayerDouble = QgsMapCanvas(self)
        self.mapViewerLayerDouble.setEnabled(False)

        self.initProject()
        self.setBrigeForMapViewer()

        self.txtMap = QApplication.translate("autoLoaderRasterPlugin","Map")
        
        self.txtGroupVectors = QApplication.translate("autoLoaderRasterPlugin","Group vectors")
        self.txtGroupRasters = QApplication.translate("autoLoaderRasterPlugin","Group rasters")
        self.txtGroupMapA = "%s A" % self.txtMap
        self.txtGroupMapB = "%s B" % self.txtMap        
        
        default = Qt.ItemIsSelectable| Qt.ItemIsEnabled | Qt.ItemIsTristate | Qt.ItemIsUserCheckable 
        defaultalr = Qt.ItemIsSelectable | Qt.ItemIsTristate 
        checkebable = Qt.ItemIsUserCheckable
        drag = Qt.ItemIsDragEnabled
        drop = Qt.ItemIsDropEnabled

        settings = {"groups":(["root"], default|drop),
                    "alrgroupA":(["root"], default|drop),
                    "alrgroupB":(["root"], defaultalr),
                    "layers":(["groups"], default|drag|drop),
                    "alrA": (["groups"], default),
                    "alrB":(["groups"], defaultalr)
                    }
            
        iconVector = None

        self.treeLegend = legendTreeWidget(settings, self.legende)
        self.treeLegend.vectors = self.treeLegend.addItem([self.txtGroupVectors], "groups", -1, self.groupIcon) 
        self.treeLegend.vectors.setCheckState(0, Qt.Unchecked)
        self.treeLegend.rasters = self.treeLegend.addItem([self.txtGroupRasters], "groups", -1, self.groupIcon) 
        self.treeLegend.rasters.setCheckState(0, Qt.Unchecked)
        self.treeLegend.alrLayersA = self.treeLegend.addItem([self.txtGroupMapA], "alrgroupA", -1, self.groupIcon) 
        self.treeLegend.alrLayersA.setCheckState(0, Qt.Checked)
        self.treeLegend.alrLayersB = self.treeLegend.addItem([self.txtGroupMapB], "alrgroupB", -1, self.groupIcon) 
        self.treeLegend.alrLayersB.setCheckState(0, Qt.Checked)        
        
        cartes = ['A', 'B']
        for carte in cartes :
            if carte == "A" : layer = self.treeLegend.addItem(["%s %s :" % (self.txtMap, carte)], "alrA" , -1, self.rasterIcon, "alrRasters", self.treeLegend.alrLayersA)
            else : layer = self.treeLegend.addItem(["%s %s :" % (self.txtMap, carte)], "alrB" , -1, self.rasterIcon, "alrRasters", self.treeLegend.alrLayersB)
            layer.setCheckState(0, Qt.Checked)
 
        self.buttonExpandAllNodes = funnyPushButton(self.legende) 
        self.buttonExpandAllNodes.initPushButton(15, 15, self.legende.width()-80, 5, "buttonExpandAllNodes", "", QApplication.translate("autoLoaderRasterPlugin", "Expand all nodes"), True, getThemeIcon("nodes_expand.png"), 15, 15, True)

        self.buttonReduceAllNodes = funnyPushButton(self.legende) 
        self.buttonReduceAllNodes.initPushButton(15, 15, self.legende.width()-60, 5, "buttonReduceAllNodes", "", QApplication.translate("autoLoaderRasterPlugin", "Close all nodes"), True, getThemeIcon("nodes_reduce.png"), 15, 15, True)

        #NEW 3.2.0
        self.closeLegend = funnyPushButton(self.legende) 
        self.closeLegend.initPushButton(15, 15, self.legende.width()-40, 5, "closeLegend", "", QApplication.translate("autoLoaderRasterPlugin", "Close"), True, getThemeIcon("closenb.png"), 15, 15, True)

        self.buttonExpandAllNodes.clicked.connect(self.treeLegend.expandTreeWidget)
        self.buttonReduceAllNodes.clicked.connect(self.treeLegend.expandTreeWidget)
        #NEW 3.2.0
        self.closeLegend.clicked.connect(self.getToolBarLegend)
        
        self.legende.setVisible(False)
        
        self.rb2 = QgsRubberBand(self.mapViewerLayerDouble, True)
        self.rbaxesh2 = QgsRubberBand(self.mapViewerLayerDouble, True)
        self.rbaxesv2 = QgsRubberBand(self.mapViewerLayerDouble, True)
        self.rb2.setColor(Qt.blue)
        self.rb2.setWidth(4)
        self.rbaxesh2.setColor(Qt.blue)
        self.rbaxesv2.setColor(Qt.blue)

        self.rbDynaMire2 = QgsRubberBand(self.mapViewerLayerDouble, True)
        self.rbDynaMire2.setColor(Qt.blue)
        self.rbDynaMire2.setWidth(1)
        self.rbDynaMire2.setLineStyle(Qt.DashDotLine) 

        self.labelCoordCursorPos = QLabel(self)
        self.labelCoordCursorPos.setStyleSheet("QLabel { font-size: 11px;  color : white; background-color: rgba(160, 160, 160, 125); border: 2px solid cyan;  border-radius: 4px;  padding: 2px;}")
        self.labelCoordCursorPos.setAlignment(Qt.AlignCenter)
        self.labelCoordCursorPos.setVisible(False)

        self.imgMagnify = QgsMapCanvas(self) 
        self.imgMagnify.setStyleSheet("QgsMapCanvas { border-width: 5px; border-radius: 10px;   border:2px solid #000000 ; background-color: rgba(255, 255, 255, 210);}")
        self.imgMagnify.setVisible(False)
        self.toolMagnifyImage = toolMagnify(self, self.mapViewerLayer, self.mapViewerLayerDouble, False)
        self.boutonMagnify.setShortcut(QKeySequence("Ctrl+M"))
        self.boutonMagnify.clicked.connect(self.setMagnify)
               

        self.mapViewerLayer.setMapTool(self.toolZoomIn)
        contextMnu = self.setContextMenu(True)
        self.mapViewerLayer.setContextMenuPolicy(Qt.CustomContextMenu)
        self.mapViewerLayer.customContextMenuRequested.connect(self.contextMnuShortCut)

        try : self.mapViewerLayer.contextMenuAboutToShow.connect(self.addContextMnu)
        except : pass
        
        self.buttonAddPermanentLayers.clicked.connect(self.addLayersToMapCanvas)
        self.buttonDelPermanentLayers.clicked.connect(self.delLayersFromMapCanvas) 

        self.iface.mapCanvas().layersChanged.connect(self.UpdateListComboLayers)        
        try : self.iface.mapCanvas().currentLayerChanged.connect(self.UpdateListComboLayers)
        except : pass
        #NEW 3.2.0
        self.iface.newProjectCreated.connect(self.clearProject) 
        self.iface.projectRead.connect(self.initContextProject) 

        QgsProject(self.iface).instance().layerRemoved.connect(self.onLayerRemoved)
        QgsProject(self.iface).instance().layersWillBeRemoved.connect(self.clearProject) 
        
        self.mapViewerLayer.extentsChanged.connect(self.synchronizeExtent)
        self.mapViewerLayer.keyPressed.connect(self.updateMagnify)

        self.listLayers.currentIndexChanged.connect(self.getUpdateMapViewers)
        self.listLayersDouble.currentIndexChanged.connect(self.setRefLayerForMapViewerDouble)
        #NEW 3.2.0
        self.initContextProject()
        self.resizeListLayers()


    #NEW 3.2.0
    def initContextProject(self):
        self.clearProject
        
        self.UpdateListComboLayers()
        self.getUpdateMapViewers()
        self.synchronizeExtent()
        self.setRefLayerForMapViewerDouble()
        self.zoomAllLayerRaster()
    
        
    def onLayerRemoved(self, layer):
        self.UpdateListComboLayers()
        self.getUpdateMapViewers()
        

    #NEW 3.2.0
    def setIndexForListLayer(self, listLayers, index):
         if self.listLayers.count() == 0 : return 0
         layer = getComboLayer(self, listLayers,  index)
         while not layer :
               index+= 1
               layer = getComboLayer(self, listLayers,  index)
         return index      

    
    def UpdateListComboLayers(self):
        self.listLayers.blockSignals(True)
        self.modelLayers = QStandardItemModel()
        indexLayer = 0
        
        if self.listLayers.model().item(self.listLayers.currentIndex(),1)!= None :
           self.comboLayersCurrentLayerId = self.listLayers.model().item(self.listLayers.currentIndex()).data(0) 

        maxdim = getListLayers(self, self.tableListLayers, self.model, self.listLayers)

        indexLayer = getComboLayerIndex(self, self.listLayers, "%s" % (self.comboLayersCurrentLayerId))
        #NEW 3.2.0
        indexLayer = self.setIndexForListLayer( self.listLayers, indexLayer)         
        self.listLayers.setCurrentIndex(indexLayer)

        self.updateListLayersDouble()
        self.listLayersDouble.setCurrentIndex(indexLayer)
        
        
        self.listLayers.blockSignals(False)
        if self.iface.mapCanvas().currentLayer()!= None:
            self.iface.mapCanvas().currentLayer().nameChanged.connect(self.UpdateListComboLayers)
            self.iface.mapCanvas().currentLayer().legendChanged.connect(self.UpdateListComboLayers)

        
    def swapListMember(self):

        if self.listLayers.count() == 0 : return
        if self.listLayersDouble.count() == 0 : return

        #NEW 3.2.0
        layer2To1 = getComboLayer(self, self.listLayers,  self.listLayersDouble.currentIndex())
        extent1 = self.mapViewerLayer.extent()
        extent2 = self.mapViewerLayerDouble.extent()

        self.listLayers.blockSignals(True)
        self.listLayersDouble.blockSignals(True)

        nameLayer, removeLayers = self.listLayers.currentText(), []

        root = self.project.layerTreeRoot()
        allLayers = root.layerOrder()
        for layer in allLayers:
            if layer.name() == nameLayer : removeLayers.append(layer.id())

        self.initMaps()
        
        index1, index2 = self.listLayers.currentIndex(), self.listLayersDouble.currentIndex()
        
        self.project.removeMapLayers(removeLayers)

        self.listLayers.setCurrentIndex(index2)
        self.listLayersDouble.setCurrentIndex(index1)
       
        self.updateMapViewers(self.listLayers)
        self.setRefLayerForMapViewerDouble()

        self.synchronizeExtent()
        self.mapViewerLayer.refreshAllLayers()

        self.mapViewerLayer.blockSignals(True)
        self.mapViewerLayerDouble.blockSignals(True)

        self.mapViewerLayer.setExtent(extent2)
        self.mapViewerLayerDouble.setExtent(extent1)        
 
        self.mapViewerLayer.blockSignals(False)
        self.mapViewerLayerDouble.blockSignals(False)
        self.listLayers.blockSignals(False)
        self.listLayersDouble.blockSignals(False)
        
        self.synchronizeMap(index1)
        self.updateMire()

       

        
        
    def updateMagnify(self, event) :
        if event.key() in (Qt.Key_Left, Qt.Key_Right, Qt.Key_Up, Qt.Key_Down) :
             cp = self.mapViewerLayer.mapTool().toMapCoordinates(self.toolMagnifyImage.mapViewer.mouseLastXY())
             self.toolMagnifyImage.imgMagnify.setCenter(QgsPointXY(cp))
             self.toolMagnifyImage.imgMagnify.refresh()
             self.toolMagnifyImage.updateRb()
        elif event.key() == Qt.Key_Escape :
             self.buttonZoomIn.clicked.emit()
             self.buttonZoomIn.setChecked(True)
             self.toolMagnifyImage.deactivate()
             self.mapViewerLayer.setMapTool(self.toolZoomIn)

        
    def keyPressEvent(self, event) :
        if self.boutonMagnify.isChecked() :
            if event.key() == Qt.Key_Tab :
                self.toolMagnifyImage.firstMapViewer = not (self.toolMagnifyImage.firstMapViewer)
                self.toolMagnifyImage.updateMapViewer()
            elif event.key() == Qt.Key_Plus : self.toolMagnifyImage.zoomInMg()
            elif event.key() == Qt.Key_Minus : self.toolMagnifyImage.zoomOutMg()

            
    def setMagnify(self) : self.mapViewerLayer.setMapTool(self.toolMagnifyImage)

    def getGridParams(self):
        sender = self.sender() 
        if sender == self.closeGridParams : sender = self.paramsGridBut
        self.setVisibility(self.widgetmapViewerLayerGridParams, sender, "gridparams_%s.png")
        if sender == self.paramsGridBut : self.paramsGrid.setIcon(self.paramsGridBut.icon())
        else : self.paramsGridBut.setIcon(self.paramsGrid.icon())
        
    def setColors(self):
        color = self.mireColor.color()

        self.rbDynaMire.setColor(color)
        self.rbDynaMire2.setColor(color)

        self.rb.setColor(color)
        self.rbaxesh.setColor(color)
        self.rbaxesv.setColor(color)
        
        self.rb2.setColor(color)
        self.rbaxesh2.setColor(color)
        self.rbaxesv2.setColor(color)

        if self.showGrids : self.refreshMaps()    


    #NEW - 3.2.0     
    def setTransformerCrs(self):
        if self.mapViewerLayer.layers() == [] or self.mapViewerLayerDouble.layers() == []: return
        self.transformer.setSourceCrs(self.mapViewerLayer.layers()[0].crs())
        self.transformer.setDestinationCrs(self.mapViewerLayerDouble.layers()[0].crs())

    def matchTransformer(self, extent, sourceProj, destProj, sensProj):
        try :
            if sourceProj != destProj : extent = self.transformer.transform(extent, sensProj)
        except : pass    
        return extent

    
    def updateMire(self) :
        self.lastExtent = self.mapViewerLayer.extent()
        #NEW - 3.2.0
        layer = getComboLayer(self, self.listLayers,  self.listLayersDouble.currentIndex())
        if layer : self.getMire(layer) 

    #NEW 3.2.0
    def resetMire(self):
        self.rb.reset()
        self.rb2.reset()           
        self.rbaxesh.reset()
        self.rbaxesv.reset()
        self.rbaxesh2.reset()
        self.rbaxesv2.reset()
        
    def getMire(self, layer):
        #NEW 3.2.0
        self.resetMire()
        if self.showGrids :
           if self.mapViewerLayer.layers() != [] :
              self.drawGrids(self.mapViewerLayer.extent(), self.rbaxesh, self.rbaxesv, self.mapViewerLayer.layers()[0].crs())
              self.drawGrids(self.mapViewerLayerDouble.extent(), self.rbaxesh2, self.rbaxesv2, self.mapViewerLayerDouble.layers()[0].crs()) #layer.crs()) 

    def drawGrids(self, extent, rbAxesX, rbAxesY, crs): 
              nRange = int(self.listParamsGrid.currentText())
              coeff = 1 / nRange
              xMin, xMax = extent.xMinimum(), extent.xMaximum()
              yMin, yMax = extent.yMinimum(),extent.yMaximum()
              
              for i in range(1, nRange, 1) :
                  meanX = extent.xMinimum() + abs(coeff * i * (extent.xMaximum()-extent.xMinimum())) 
                  meanY = extent.yMinimum() + abs(coeff * i * (extent.yMinimum()-extent.yMaximum()))
                  point1, point2, point3, point4 = QgsPointXY(xMin, meanY), QgsPointXY(xMax, meanY), QgsPointXY(meanX, yMin), QgsPointXY(meanX, yMax)                  
                  geometry = QgsGeometry().fromPolylineXY([point1, point2])
                  rbAxesX.addGeometry(geometry, crs) 
                  geometry = QgsGeometry().fromPolylineXY([point3, point4])
                  rbAxesY.addGeometry(geometry, crs) 
             

    def getToolBarNav(self): self.setVisibility(self.widgetMapView, self.buttonToolBarNav, "navigation_%s.png")
    def getToolBarLegend(self): self.setVisibility(self.legende, self.buttonToolBarLegend, "legend_%s.png")
    
    def updateGridMaps(self, index):
        if self.showGrids : self.updateMire()
        if self.boutonMagnify.isChecked() : self.boutonMagnify.setFocus()
        
    def getGridMaps(self):
        self.showGrids = not( self.showGrids)
        self.buttonToolBarGrid.setIcon(QIcon(getThemeIcon("grid_%s.png" % iff(self.showGrids, "on", "off"))))
        self.updateMire()
        
    
    def setVisibility(self, widget, actionTool, icon):
        if widget :
           widget.setVisible(not widget.isVisible ())
           cond = "on" if widget.isVisible() else "off"
        else : cond = "off"
        actionTool.setIcon(QIcon(getThemeIcon(icon % cond)))


    #NEW 3.2.0
    def resizeListLayers(self):
        centerX = int(self.width()/2)        
        self.listLayers.setGeometry(5, 5, centerX-5-140, 25)
        self.listLayersDouble.setGeometry(centerX+15, 5, centerX-20, 25)
        dim = max(self.listLayers.width(), self.listLayersDouble.width())
        
        self.view.horizontalHeader().setDefaultSectionSize(dim)
        self.view.setStyleSheet("QTableView {min-width: %spx; max-width: %spx; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}" % (dim, dim))

        self.view2.horizontalHeader().setDefaultSectionSize(dim)
        self.view2.setStyleSheet("QTableView {min-width: %spx; max-width: %spx; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}" % (dim, dim))


    def resizeEvent(self, event):
        centerX = int(self.width()/2)
        #NEW 3.2.0
        self.resizeListLayers()
        
        self.buttonToolBarNav.setGeometry(centerX-130, 5, 25, 25) 
        self.buttonToolBarLegend.setGeometry(centerX-100, 5, 25, 25) 
        self.buttonToolBarGrid.setGeometry(centerX-75, 5, 25, 25) 
        self.buttonActions.setGeometry(centerX-55, 5, 25, 25) 
        self.buttonReverse.setGeometry(centerX-10, 5, centerX-5, 25)
        
        self.mapViewerLayer.setGeometry(5, 30, centerX-5, self.height()-40)
        #NEW 3.2.0
        self.barInfo.setGeometry(0, self.mapViewerLayer.height()-90, self.mapViewerLayer.width(), 70)
        self.barInfo.resize(self.mapViewerLayer.width(), 70)
        
        self.labelScale.setGeometry(self.mapViewerLayer.width()-110, self.height()-35, 100, 20)
        self.mapViewerLayerDouble.setGeometry(centerX, 30, centerX-5, self.height()-40)

        #NEW 3.2.0
        self.updateMire()


    def setPropertiesLayer(self, layer, item): 

        self.mapViewerLayer.setCurrentLayer(layer)
        
        if not layer : return
        
        if type(layer) == QgsVectorLayer :
            dlg = QgsRendererPropertiesDialog(layer, QgsStyle.defaultStyle(), False)
            dlg.item = item
            dlg.layer = layer

            groupBox = dlg.findChild(QGroupBox, "mLayerRenderingGroupBox")
            if groupBox :
               opacityWidget = groupBox.findChild(QWidget, "mOpacityWidget")
               if opacityWidget :
                  for child in opacityWidget.children() :
                      if type(child) == QSlider : dlg.slider = child  ; break
               
                            
            
        elif type(layer) == QgsRasterLayer : 
            dlg = QDialog()
            dlg.item = item
            dlg.layer = layer
            dlg.setMaximumWidth(800)
            dlg.setMaximumHeight(800)
            dlg.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowStaysOnTopHint | Qt.WindowCloseButtonHint)
            dlg.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin", "Properties for the layer"))
            dlg.setGeometry(200, 200, 600, 600)
            layout = QVBoxLayout(dlg)

            tabs = QTabWidget(dlg) 
            tabs.setTabsClosable(False)
            tabs.setDocumentMode(True)
            tabs.setTabShape(QTabWidget.Rounded)

            icon = getThemeIcon("raster.png")

            scrollAreaA = QScrollArea(dlg)
            dlg.widgetproperties = QgsRendererRasterPropertiesWidget(layer, self.mapViewerLayer, dlg)
            dlg.widgetproperties.setGeometry(10, 10, 580, 580)
            scrollAreaA.setWidget(dlg.widgetproperties)
            
            tabs.addTab(scrollAreaA, QIcon(icon), QApplication.translate("autoLoaderRasterPlugin", "Renderer's params"))

            dlg.slider = None
            scrollAreaB = QScrollArea(dlg)
            dlg.transparencyProperties = QgsRasterTransparencyWidget(layer, self.mapViewerLayer, dlg)

            dlg.slider = None
            opacityWidget = dlg.findChild(QWidget, "mOpacityWidget")
            if opacityWidget :
               for child in opacityWidget.children() :
                   if type(child) == QSlider : dlg.slider = child ; break
                               
            dlg.transparencyProperties.setGeometry(10, 10, 560, 580)
            scrollAreaB.setWidget(dlg.transparencyProperties)
            
            tabs.addTab(scrollAreaB, QIcon(icon), QApplication.translate("autoLoaderRasterPlugin", "Opacity and Transparency"))

            layout.addWidget(tabs)
           
        
        else : return
        
        windowIcon(dlg , "layers.png")
        dlg.setWindowTitle("%s : %s" % (dlg.windowTitle(), layer.name()))
        buttonBox = None
        for conteneur in dlg.children():
                if conteneur.children()!= []:
                   for child in conteneur.children():
                          for elt in child.children():
                                  if type(elt) == QDialogButtonBox :
                                             buttonBox = elt
                                             break

        
        if buttonBox and layer.type() == 0 :
                self.applystyle = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin", "Apply style"), QDialogButtonBox.ActionRole)
                self.applystyle.clicked.connect(self.apply_style)
                buttonBox.accepted.connect(self.apply_style_accept)
                buttonBox.rejected.connect(dlg.close)
        else :
           buttonBox = QDialogButtonBox()
           self.quitButton = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin","Cancel"), QDialogButtonBox.RejectRole)
           self.applystyle = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin", "Apply style"), QDialogButtonBox.ActionRole)
           self.acceptButton = buttonBox.addButton(QApplication.translate("autoLoaderRasterPlugin","Validate"), QDialogButtonBox.AcceptRole)
           buttonBox.accepted.connect(self.apply_style_accept)
           buttonBox.rejected.connect(dlg.reject)
           layout.addWidget(buttonBox)
           self.applystyle.clicked.connect(self.apply_style)

        self.styleDlg = dlg                          
        dlg.exec_()
        self.mapViewerLayer.refresh()


    def setSliderValue(self):
        if self.styleDlg :
            item = self.styleDlg.item
            if self.styleDlg.slider :
               self.treeLegend.setSlider(item, 100- (self.styleDlg.slider.value()/10))

    
    def apply_style_accept(self) :
        self.apply_style()
        self.styleDlg.accept()
          

    def apply_style(self):
        if self.styleDlg:
           if self.mapViewerLayer.currentLayer().type()==0 :
              self.styleDlg.apply()
              self.setSliderValue()
           else :
              self.styleDlg.widgetproperties.apply()
              self.styleDlg.transparencyProperties.apply()
              self.setSliderValue()
           self.refreshMaps()


    def clearProject(self):
        self.layers, self.currentLayers = [], []

        self.mapViewerLayer.blockSignals(True)

        if self.project : self.project.addMapLayers([])
        self.listLayers.clear()
        #NEW 3.2.0
        self.listLayersDouble.clear()
        self.initProject()
        #NEW 3.2.0
        self.resetMire()
        self.initMaps()
        
        self.mapViewerLayer.blockSignals(False)

        
        
    def initProject(self):
        self.project = QgsProject(self)
        crs = QSettings().value("Projections/layerDefaultCrs")
        self.crs = QgsCoordinateReferenceSystem(crs)
        #NEW 3.2.0
        if self.mapViewerLayer.layers() != [] :
           self.crs = self.mapViewerLayer.layers()[0].crs()
        self.project.setCrs(self.crs)
        self.project.addMapLayers([])

    def setBrigeForMapViewer(self) :  self.bridgeA = QgsLayerTreeMapCanvasBridge(self.project.layerTreeRoot(), self.mapViewerLayer)

    def contextMnuZoomAll(self, button):
        contextMenu = QMenu()

        #NEW 3.2.0
        action = QAction(QIcon(getThemeIcon("zoomtolayer.png")), "%s...%s A" % (QApplication.translate("autoLoaderRasterPlugin", "Zoom to raster layer extent"),
                                                                                QApplication.translate("autoLoaderRasterPlugin", "Map")), self)
        action.setObjectName("zoomallrasterlayerA")
        contextMenu.addAction(action)
        setattr(self, "zoomallrasterlayerA", action)
        action.triggered.connect(self.zoomAllLayerRaster)
        
        action = QAction(QIcon(getThemeIcon("zoomtolayer.png")), "%s...%s B" % (QApplication.translate("autoLoaderRasterPlugin", "Zoom to raster layer extent"),
                                                                                QApplication.translate("autoLoaderRasterPlugin", "Map")), self)

        action.setObjectName("zoomallrasterlayerB")
        contextMenu.addAction(action)
        setattr(self, "zoomallrasterlayerB", action)
        action.triggered.connect(self.zoomAllLayerRaster)

        contextMenu.addSeparator()

        action = QAction(QIcon(getThemeIcon("zoommap.png")), "%s A" % QApplication.translate("autoLoaderRasterPlugin", "Zoom all the map window"), self)
        action.setObjectName("zoomallmapA")
        contextMenu.addAction(action)
        setattr(self, "zoomallmapA", action)
        action.triggered.connect(self.zoomAllMap)

        action = QAction(QIcon(getThemeIcon("zoommap.png")), "%s B" % QApplication.translate("autoLoaderRasterPlugin", "Zoom all the map window"), self)
        action.setObjectName("zoomallmapB")
        contextMenu.addAction(action)
        setattr(self, "zoomallmapB", action)
        action.triggered.connect(self.zoomAllMap)         

        button.setMenu(contextMenu)
        

    def refreshContext(self):
        self.treeLegend.removeAllLayers()
        self.initProject()
        self.updateMapViewers(self.listLayers)
        self.updateListLayersDouble()
        self.setBrigeForMapViewer()
        self.synchronizeExtent()

        
    def contextMnuActions(self, button):  self.getContextMnu(self.sender(), None, button)
    def contextMnuShortCut(self, point): self.getContextMnu(self.sender(), point, None)
    def getContextMnu(self, sender, point, button):  
        if point :
          if not (sender == self.mapViewerLayer) : return
        contextMnu = self.setContextMenu(False) if button else self.setContextMenu(True)
        if point : contextMnu.exec_(self.sender().mapToGlobal(point))
        else : button.setMenu(contextMnu) 
         
    def setContextMenu(self, fromMap):
        if fromMap == True :
            
            self.toolMagnifyImage.deactivate()
            self.mapViewerLayer.setMapTool(self.toolZoomIn)
            self.buttonZoomIn.setChecked(True)
            
            actions = {"paramsGrid" : ("gridparams_%s.png", "Parameters for the grid", "" , self.getGridParams, True, "Ctrl+O"),
                       "~0" : (),  
                       "prinMapAsImages" : ("print.png", "Print the maps", "" , self.printMaps, True, "Ctrl+P"),
                       "saveMapAsImages" : ("savefile.png", "Save maps as images", "" , self.saveMapsAsImage, True, "Ctrl+S"),
                       "~1" : (),
                       "reloadContextAction" : ("reload.png", "Reload QGIS context", "" , self.refreshContext, True, "Ctrl+Q")
                       }
        else :
            actions = {"paramsGridBut" : ("gridparams_off.png", "Parameters for the grid", "" , self.getGridParams, True, "Ctrl+O"),
                       "~0" : (),  
                       "prinMapAsImages" : ("print.png", "Print the maps", "" , self.printMaps, True, "Ctrl+P"),
                       "saveMapAsImages" : ("savefile.png", "Save maps as images", "" , self.saveMapsAsImage, True, "Ctrl+S"),
                       "~1" : (),
                       "reloadContext" : ("reload.png", "Reload QGIS context", "" , self.refreshContext, True, "Ctrl+Q")
                       }          
         
        listItems = []
        for key in actions.keys() :
            if not key.startswith("~") : listItems.append(QApplication.translate("autoLoaderRasterPlugin", actions[key][1]).replace(":",""))
        maxDim = fixeMaxDim(listItems)

        contextMnu = QMenu() 
        for key in actions :
            if key.startswith("~") : contextMnu.addSeparator()
            else :
                  icon = actions[key][0]
                  if key == "paramsGrid" :
                     cond = "on" if self.widgetmapViewerLayerGridParams.isVisible() else "off"
                     icon = getThemeIcon(icon % cond)
                  else : icon = getThemeIcon(icon)

                  text = textForMenu("%s...%s", QApplication.translate("autoLoaderRasterPlugin", actions[key][1]).replace(":",""), maxDim, actions[key][5])
                  action = QAction(QIcon(icon), text, self)
                  
                  if actions[key][2] != "" : action.setShortcut(QKeySequence(actions[key][2]))
                  action.setEnabled(actions[key][4])
                  action.setObjectName(key)
                  action.setShortcut(QKeySequence(actions[key][5]))
                  contextMnu.addAction(action)
                  action.triggered.connect(actions[key][3])

                  setattr(self, key, action)

        return contextMnu


    def addContextMnu(self, mapCanvasContextMnu) :
        contextMnu = self.setContextMenu(True)
        mapCanvasContextMnu.addMenu(contextMnu)
        

    def saveMapsAsImage(self):
        dlgFile = saveDlg(self, self.initDir)
        dlgFile.exec_()

    def createFilesImages(self, folder):
        fileName = join(folder, "mapviewer1.png") 
        self.mapViewerLayer.saveAsImage(fileName, None, "PNG")

        fileNameDouble = join(folder ,"mapviewer2.png") 
        self.mapViewerLayerDouble.saveAsImage(fileNameDouble, None, "PNG")

        return fileName, fileNameDouble
       
    def printMaps(self):
        printer = QPrinter(); printer.setPageSize(QPrinter.A4); printer.setOrientation(QPrinter.Landscape);
        printer.setPageMargins(5, 5, 5, 5, QPrinter.Millimeter); printer.setOutputFormat(QPrinter.NativeFormat);

        fileName, fileNameDouble = self.createFilesImages(join(dirname(__file__), "works"))
       
        self.printMapview.clear()
        self.printMapview.setPageSize(QSizeF(printer.pageRect().size()))
       

        try:
            from osgeo.gdal import Open as gdalopen
            from osgeo.gdal import GA_ReadOnly
        except:
            from gdal import Open as gdalopen
            from gdal import GA_ReadOnly

        sourceWidth, sourceHeight = printer.pageRect().width(), printer.pageRect().height()
        dataset = gdalopen( fileName, GA_ReadOnly )
        orientation = QPrinter.Landscape
        
        if dataset :
            imgWidth, imgHeight = dataset.RasterXSize, dataset.RasterYSize
            if imgHeight > (imgWidth*1.1) : orientation = QPrinter.Portrait


        if orientation == QPrinter.Portrait :  scale = sourceHeight*0.8 / float(imgHeight)
        else :  scale = sourceWidth / float(imgWidth*2)

        
        imgWidth, imgHeight = int(imgWidth*scale), int(imgHeight*scale)

        orientation = QPrinter.Landscape
       
        title = "%s : " % QApplication.translate("autoLoaderRasterPlugin", "Print")

        html = """
        <head><title>Report</title></head>
        <body>
            <table width="100%">
                <tr><td><span style='font-size:1.2em'>{}</span><br><small>{} [{}]<br>{} [{}]</small></td></tr>
            </table>
          <hr>
          <table width="100%">
          <tr>
            <td><img src="{}" width="{}" height="{}"></td>
            <td><img src="{}" width="{}" height="{}"></td>
          </tr>
          </table>
          <hr>
          <p align=right>AutoLaoderRaster, le {}</p>
        </body>
        """.format(title, fileName, self.listLayers.currentText(), fileNameDouble, self.listLayersDouble.currentText(), fileName, imgWidth, imgHeight, fileNameDouble, imgWidth, imgHeight, thedatetime.now().strftime("%d/%m/%Y à %Hh%M"))
       
        self.printMapview.setHtml(html)        
        dlgPrint = printerDlg(self, self.printMapview, "%s : " % QApplication.translate("autoLoaderRasterPlugin", "Print"), orientation)


    def catchExtent(self):
        indexListLayers = self.listLayers.currentIndex()
        
        extent = self.canvas.extent()
        coords = [[ QgsPointXY(extent.xMinimum(),extent.yMaximum()), QgsPointXY(extent.xMaximum(),extent.yMaximum()),
                                  QgsPointXY(extent.xMaximum(),extent.yMinimum()), QgsPointXY(extent.xMinimum(),extent.yMinimum()),
                                  QgsPointXY(extent.xMinimum(),extent.yMaximum()), ]]  
        geomTargetExtent = QgsGeometry().fromPolygonXY(coords)

        overLaps, favExtentIndex, indexes = 0, 0, []
        
        for i in range(self.listLayers.count()):
              layer = getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())              
              if not layer : return               
              extent = layer.extent()
              coords = [[ QgsPointXY(extent.xMinimum(),extent.yMaximum()), QgsPointXY(extent.xMaximum(),extent.yMaximum()),
                                  QgsPointXY(extent.xMaximum(),extent.yMinimum()), QgsPointXY(extent.xMinimum(),extent.yMinimum()),
                                  QgsPointXY(extent.xMinimum(),extent.yMaximum()), ]]  
              geomExtent = QgsGeometry().fromPolygonXY(coords)

              if geomExtent.intersects(geomTargetExtent) :
                  area = geomExtent.intersection(geomTargetExtent).area()
                  if area > 0 :
                     indexes.append(i)
                     if area > overLaps : overLaps, favExtentIndex = area, i
 
        self.listLayers.setCurrentIndex(indexListLayers) if indexListLayers in indexes else self.listLayers.setCurrentIndex(favExtentIndex)
        self.getUpdateMapViewers()
        self.mapViewerLayer.setExtent(self.canvas.extent())
        
    #NEW 3.2.0
    def sendExtentToMapCanvas(self):
        if self.mapViewerLayer.layers() == [] : return
        try :
           extent = self.mapViewerLayer.extent() 
           if self.mapViewerLayer.layers()[0].crs() != QgsProject(self.iface).instance().crs() :
              transformer = QgsCoordinateTransform()
              transformer.setSourceCrs(self.mapViewerLayer.layers()[0].crs())
              transformer.setDestinationCrs(QgsProject(self.iface).instance().crs())
              extent = transformer.transform(extent, QgsCoordinateTransform.ForwardTransform)            
           self.canvas.setExtent(extent)
           self.canvas.refresh()
        except : pass

    #NEW 3.2.0
    def initMaps(self):
        self.mapViewerLayer.setLayers([])
        self.mapViewerLayerDouble.setLayers([])
        self.mapViewerLayer.clearCache()
        self.refreshMaps() 
    
    def refreshMaps(self) :
        self.mapViewerLayer.refresh()
        self.mapViewerLayerDouble.refresh()   

    def zoomIn(self): self.mapViewerLayer.setMapTool(self.toolZoomIn) 
    def zoomOut(self): self.mapViewerLayer.setMapTool(self.toolZoomOut) 


    #NEW 3.2.0 
    def zoomAndSaveContext(self):
        if self.mapViewerLayer.layers() != [] :

           oldExtent = self.mapViewerLayer.extent()
           crs = self.mapViewerLayer.layers()[0].crs()

           layer = getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())
           if not layer : return None

           if layer : 
              if layer.crs() != crs :
                  transformer = QgsCoordinateTransform()
                  transformer.setSourceCrs(crs)
                  transformer.setDestinationCrs(layer.crs())
                  oldExtentProj = transformer.transform(oldExtent, QgsCoordinateTransform.ForwardTransform)
              else : oldExtentProj = oldExtent
           else : oldExtentProj = oldExtent  

           if layer :
               oldGeomExtent = QgsGeometry().fromRect(oldExtentProj) if layer.crs() != crs else QgsGeometry().fromRect(oldExtent)
           else : oldGeomExtent = QgsGeometry().fromRect(oldExtent)    
           newGeomExtent = QgsGeometry().fromRect(layer.extent())    
   
           if newGeomExtent.intersects(oldGeomExtent): return oldExtentProj
           else : return layer.extent()

        return None

    #NEW 3.2.0
    def zoomAllLayerRaster(self):
        if self.sender() == self.zoomallrasterlayerB :
           self.mapViewerLayer.blockSignals(True) 
           self.setMaps2Extent()
           self.mapViewerLayer.blockSignals(False)
           self.synchronizeExtent()
        else :
           layer = getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())
           if layer : self.mapViewerLayer.setExtent(layer.extent())
           
        self.updateMire()
        self.updateLabelScale()
            
    def zoomAllMap(self):
        #NEW 3.2.0
        if self.sender()== self.zoomallmapB :
           self.mapViewerLayer.blockSignals(True)
           self.mapViewerLayerDouble.zoomToFullExtent() 
           self.mapViewerLayer.setExtent(self.mapViewerLayerDouble.extent())
           self.refreshMaps()
           self.updateLabelScale()
           self.mapViewerLayer.blockSignals(False)
        else : self.mapViewerLayer.zoomToFullExtent()
        self.updateMire()
          
    def zoomPrevious(self): self.mapViewerLayer.zoomToPreviousExtent()
    def zoomNext(self): self.mapViewerLayer.zoomToNextExtent()
    def pan(self): self.mapViewerLayer.setMapTool(self.toolPan)         


    #NEW 3.2.0
    def setMaps2Extent(self):
        #try :         
        layerMapB = getComboLayer(self, self.listLayers,  self.listLayersDouble.currentIndex())
        extent = layerMapB.extent()
        self.mapViewerLayerDouble.setExtent(extent)
        self.mapViewerLayerDouble.setDestinationCrs(layerMapB.crs())

        if self.mapViewerLayer.layers() != [] :
           layerMapA = getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())
           self.mapViewerLayer.setDestinationCrs(layerMapA.crs())           
           self.setTransformerCrs()
           if (layerMapA.crs() != layerMapB.crs()) :             
               extent = self.matchTransformer(extent,  layerMapA.crs(), layerMapB.crs(), QgsCoordinateTransform.ReverseTransform)
        self.mapViewerLayer.setExtent(extent)        
        self.synchronizeMap(self.listLayersDouble.currentIndex())
                    
        #except : pass    
  
    def setMapsExtent(self):        
        try :
            #NEW 3.2.0 
            layer = getComboLayer(self, self.listLayers,  self.listLayersDouble.currentIndex())
            extent = self.mapViewerLayer.extent()
            if self.mapViewerLayer.layers() != [] :
               self.setTransformerCrs()
               if (self.mapViewerLayer.layers()[0].crs() != layer.crs()) : 
                   extent = self.matchTransformer(extent, self.mapViewerLayer.layers()[0].crs(), layer.crs(), QgsCoordinateTransform.ForwardTransform)
            self.mapViewerLayerDouble.setExtent(extent)
            self.mapViewerLayerDouble.refresh()
        except : pass

    def addLayersByGroup(self, layer):        
        groupForTheLayer = self.getGroupForTheLayer(layer)
        mapLayers = self.mapViewerLayerDouble.layers()
        #NEW 3.2.0 
        refGeom = QgsGeometry().fromRect(layer.extent())
        if groupForTheLayer :
            for child in groupForTheLayer.findLayers():
                if not child.layer() in mapLayers :
                    #NEW 3.2.0
                    extGeom = QgsGeometry().fromRect(child.layer().extent())
                    if extGeom.touches(refGeom): mapLayers.insert(len(mapLayers), child.layer()) #-1, child.layer())
        self.mapViewerLayerDouble.setLayers(mapLayers)    

    def getUpdateMapViewers(self):
        #NEW 3.2.0         
        extent = self.zoomAndSaveContext()
        self.updateMapViewers(self.sender())              
        
        if not extent : self.zoomAllLayerRaster()
        else : self.mapViewerLayer.setExtent(extent)

        self.setRefLayerForMapViewerDouble() 

   
    def setRefLayerForMapViewerDouble(self):
        if self.listLayersDouble.count() == 0: return
        #NEW 3.2.0
        self.listLayersDouble.setToolTip("")        
        
        layer = getComboLayer(self, self.listLayers,  self.listLayersDouble.currentIndex())       
        if layer :
            #NEW 3.2.0
            self.listLayersDouble.setToolTip(layer.crs().authid())
            
            self.setLayers(layer)
            groupForTheLayer = self.getGroupForTheLayer(layer)
            self.treeLegend.setDataItem(self.txtGroupMapB, "%s B" % self.txtMap, "%s B : %s" % (self.txtMap, layer.name()), layer.id(), self.rasterIcon, "alrB" , "alrRasters", self.treeLegend.alrLayersB)
            self.treeLegend.setMapLayers()
            self.addLayersByGroup(layer)

            #NEW 3.2.0
            self.setMapsExtent()
            self.setTransformerCrs()
            self.updateMire()


    def updateMapViewers(self, mapLayerComboBox):
        if mapLayerComboBox == self.listLayersDouble : target, carte = self.mapViewerLayerDouble, "B"  
        else : mapLayerComboBox = self.listLayers; target, carte = self.mapViewerLayer, "A"
        #NEW 3.2.0
        mapLayerComboBox.setToolTip("")

        if not target : return
        if mapLayerComboBox.count() == 0 : return      

        layer = getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())
        #NEW 3.2.0
        if layer : mapLayerComboBox.setToolTip(layer.crs().authid())
        clayer = None
        
        if not layer in self.mapViewerLayer.layers() :
           if isinstance(layer, QgsRasterLayer) :
              clayer = layer.clone()     
              self.project.addMapLayer(clayer)
        else : clayer = layer    
        #DELETE OLD REF LAYER BEFORE
        #GET LAYER ID FOR MAP A
        group = self.treeLegend.getGroupParent("%s A" % self.txtMap)        
        if group.childCount() > 0 :
           item = group.child(0)
           oldRefLayer = self.treeLegend.layerItem(item.whatsThis(0))
           if oldRefLayer : self.project.removeMapLayers( [oldRefLayer.id()] )

        #NEW 3.2.0
        if not clayer : return   
        
        if clayer :
            self.setLayers(clayer)
            self.treeLegend.setDataItem(self.txtGroupMapA, "%s A" % self.txtMap, "%s A : %s" % (self.txtMap, clayer.name()), clayer.id(), self.rasterIcon, "alrA" , "alrRasters", self.treeLegend.alrLayersA)
            self.treeLegend.setMapLayers()

        self.setBrigeForMapViewer()

    #NEW 3.2.0
    def updateLabelScale(self):
        widthMap = convertSize(round(self.mapViewerLayer.extent().xMaximum() - self.mapViewerLayer.extent().xMinimum(), 2))        
        self.labelScale.setText("%s%s" % (widthMap, QgsUnitTypes.toString(self.refCrs.mapUnits()))) 

    def synchronizeExtent(self):
        index  = self.listLayers.currentIndex()
        self.mapViewerLayer.refresh()
        self.setMapsExtent()
        self.synchronizeMap(index)
        #NEW 3.2.0
        self.updateLabelScale()
        self.updateMire()
        if self.mapViewerLayer.mapTool() == self.toolMagnifyImage : self.toolMagnifyImage.updateRb()


    def updateListLayersDouble(self):
          #NEW 3.2.0
          self.listLayersDouble.blockSignals(True)
          if self.listLayersDouble.model().item(self.listLayersDouble.currentIndex(),1)!= None :
             self.comboLayersCurrentLayerId = self.listLayersDouble.model().item(self.listLayersDouble.currentIndex()).data(0) 

          maxdim = getListLayers(self, self.tableListLayers, self.model2, self.listLayersDouble)

          self.listLayersDouble.setEditable(False)
          self.listLayersDouble.blockSignals(False)
          
          self.synchronizeMap(self.listLayers.currentIndex())

    #NEW 3.2.0
    def getGeomFromExtent(self, extent):
        coords = [[ QgsPointXY(extent.xMinimum(),extent.yMaximum()), QgsPointXY(extent.xMaximum(),extent.yMaximum()),
                    QgsPointXY(extent.xMaximum(),extent.yMinimum()), QgsPointXY(extent.xMinimum(),extent.yMinimum()),
                    QgsPointXY(extent.xMinimum(),extent.yMaximum()), ]]  
        geomTargetExtent = QgsGeometry().fromPolygonXY(coords)
        return geomTargetExtent
  
    def synchronizeMap(self, indexTarget):
          if self.listLayers.count() == 0 : return
          #NEW 3.2.0
          if self.mapViewerLayer.layers() == [] : return
                
          self.layers, refLayer = [], None

          for i in range(self.listLayers.count()):
              layer = getComboLayer(self, self.listLayers, i)
              if layer : self.listLayersDouble.setItemData(i, 0, Qt.UserRole-1)

          geomTargetExtent = self.getGeomFromExtent(self.mapViewerLayer.extent())

          first = True
          newIndex = 0 if self.listLayersDouble.currentIndex() == -1 else self.listLayersDouble.currentIndex()
          ajustIndex, indexesGroupEnabled = False, []
          mapLayer = getComboLayer(self, self.listLayers, self.listLayers.currentIndex())

          if not mapLayer : return            
          groupForTheLayer = self.getGroupForTheLayer(mapLayer)

          #NEW 3.2.0
          refCrs = self.mapViewerLayer.layers()[0].crs()
          extent = self.mapViewerLayer.extent()
          transformer = QgsCoordinateTransform()
          transformer.setDestinationCrs(refCrs)

          
          for i in range(self.listLayers.count()):
              self.listLayersDouble.setItemText(i, self.listLayers.itemText(i))
              layer = getComboLayer(self, self.listLayers, i)
              item = self.listLayersDouble.model().item(i, 0)

              #NEW 3.2.0 - TO FINISH
              if layer :
                 if layer.extent() :
                    extent = layer.extent()
                    if layer.crs() != refCrs :
                       transformer.setSourceCrs(layer.crs())
                       extentProj = transformer.transform(extent, QgsCoordinateTransform.ForwardTransform)
                       extent = extentProj
                    geomExtent = self.getGeomFromExtent(extent)                    
                    
                    if geomExtent.intersects(geomTargetExtent) :
                  
                        try : self.listLayersDouble.setItemData(i, 33, Qt.UserRole-1)
                        except : return
                        
                        if first :  refLayer = layer; first = False 
                        if i == indexTarget : refLayer = layer 
                        self.layers.append(layer)

                        if self.getGroupForTheLayer(layer) == groupForTheLayer : indexesGroupEnabled.append(i) 

                    else : self.listLayersDouble.setItemData(i, 0, Qt.UserRole-1)
       
                     
              if i == newIndex :
                 if item : 
                    if not item.isEnabled() :  ajustIndex = True  

          if ajustIndex :
             if indexesGroupEnabled != []: newIndex = indexesGroupEnabled[0]

          self.setLayers(refLayer)



    def getVectorLayer(self):      
        root = QgsProject.instance().layerTreeRoot()
        allLayers, allLayersIcon = [], []
        iconVector, iconRaster = QIcon(getThemeIcon("vector.png")),  QIcon(getThemeIcon("raster.png"))
        for layer in root.findLayers() : #TO FINISH - wrapped C/C++ object of type QgsRasterLayer has been deleted
            if type(layer.layer()) in (QgsVectorLayer, QgsRasterLayer) :
               allLayers.append(layer.layer().name())
               allLayersIcon.append(iconVector if layer.layer().type() == 0 else iconRaster)
        return allLayers, allLayersIcon

    def delLayersFromMapCanvas(self):
        frozenLayer = getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())

        allLayers, allLayersIcon = [], []
        iconVector, iconRaster = QIcon(getThemeIcon("vector.png")),  QIcon(getThemeIcon("raster.png"))

        for layer in self.mapViewerLayer.layers() : 
            if type(layer) in (QgsVectorLayer, QgsRasterLayer) :
               if not layer.name() in allLayers :
                  allLayers.append(layer.name())
                  allLayersIcon.append(iconVector if layer.type() == 0 else iconRaster)
        if allLayers == [] : return
        
        self.dialogInput = QInputDialog(self) 
        self.dialogInput.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin","Resources"))
        self.dialogInput.setLabelText("%s :" % QApplication.translate("autoLoaderRasterPlugin","Layers"))
        self.dialogInput.setOptions(QInputDialog.UseListViewForComboBoxItems)
        windowIcon(self.dialogInput, "del.png")

        self.dialogInput.resize(600, 500)
        self.dialogInput.setComboBoxItems(allLayers) 

        child = None
        for child in self.dialogInput.children():
            if type(child) == QListView :
                child.setIconSize(QSize(18, 18))
                child.setSelectionMode( QAbstractItemView.ExtendedSelection )

                child.doubleClicked.connect(self.dialogInput.accept)                

                model = child.model()
                for index in range(model.rowCount()):
                    listViewItem = model.item(index)
                    listViewItem.setData( allLayersIcon[index], Qt.DecorationRole)
                    if frozenLayer :
                       if listViewItem.text() == frozenLayer.name() : listViewItem.setEnabled(False)
       

        if self.dialogInput.exec_() :
            if child :
               instance = self.project
               layersMapView = self.mapViewerLayer.layers()
               layersMapViewDouble = self.mapViewerLayerDouble.layers()
               
               for index in child.selectionModel().selectedIndexes() :
                   nameLayer = child.model().itemFromIndex(index).text()
                   layers = instance.mapLayersByName(nameLayer)
                   
                   for layer in layers :
                      groupText = self.txtGroupRasters if type(layer) == QgsRasterLayer else self.txtGroupVectors #"Autres raster" "Groupe vecteur"
                      self.treeLegend.removeItemWithId(groupText, layer.id())
                       
                      if layer in layersMapView : layersMapView.remove(layer)
                      if layer in layersMapViewDouble :  layersMapViewDouble.remove(layer)
                                            
                      self.project.removeMapLayers( [layer.id()] )

               self.afterAddOrDelLayers()
   
          
    def addLayersToMapCanvas(self):
        
        frozenLayer =  getComboLayer(self, self.listLayers,  self.listLayers.currentIndex())
        allLayers, allLayersIcon = self.getVectorLayer()
        if allLayers == [] : return
        
        self.dialogInput = QInputDialog(self) 
        self.dialogInput.setWindowTitle(QApplication.translate("autoLoaderRasterPlugin","Resources"))
        self.dialogInput.setLabelText("%s :" % QApplication.translate("autoLoaderRasterPlugin","Layers"))
        self.dialogInput.setOptions(QInputDialog.UseListViewForComboBoxItems)
        windowIcon(self.dialogInput, "add.png")

        self.dialogInput.resize(600, 500)
        
        self.dialogInput.setComboBoxItems(allLayers) 
        icon = QIcon(getThemeIcon("vector.png"))

        child = None
        for child in self.dialogInput.children():
            if type(child) == QListView :
 
                child.doubleClicked.connect(self.dialogInput.accept)
                
                child.setIconSize(QSize(18, 18))
                child.setSelectionMode( QAbstractItemView.ExtendedSelection )

                model = child.model()
                for index in range(model.rowCount()):
                    listViewItem = model.item(index)
                    listViewItem.setData( allLayersIcon[index], Qt.DecorationRole)
                    if frozenLayer :
                       if listViewItem.text() == frozenLayer.name() : listViewItem.setEnabled(False)

        if self.dialogInput.exec_() :
            
            if child :
               instance = QgsProject.instance()
               
               for index in child.selectionModel().selectedIndexes() :
                   nameLayer = child.model().itemFromIndex(index).text()
                   layer = instance.mapLayersByName(nameLayer)
                  
                   if layer : 
                      if type(layer)== list : layer = layer[0]

                      if  not layer in self.mapViewerLayer.layers() :
                          
                          clayer = layer.clone()  
                          if layer.hasScaleBasedVisibility() : clayer.setScaleBasedVisibility(False)
                          self.project.addMapLayer(clayer)

                          layer = self.treeLegend.addItem([clayer.name()], "layers", clayer.id(), self.rasterIcon, "rasters", self.treeLegend.rasters) if type(clayer) == QgsRasterLayer else  self.treeLegend.addItem([clayer.name()],  "layers", clayer.id(), self.vectorIcon, "vectors", self.treeLegend.vectors)
                          layer.setCheckState(0, Qt.Checked)

               self.afterAddOrDelLayers()

    def afterAddOrDelLayers(self):                          
        extent = self.zoomAndSaveContext() #self.mapViewerLayer.extent()
        self.mapViewerLayer.blockSignals(True)
        self.setBrigeForMapViewer()               
        self.setRefLayerForMapViewerDouble() 
        self.mapViewerLayer.blockSignals(False)
        #NEW 3.2.0
        self.zoomallrasterlayerB.triggered.emit()
        if self.mapViewerLayer.mapTool() == self.toolMagnifyImage : self.toolMagnifyImage.updateMapViewer()
                   
    def setLayers(self, refLayer):
        self.currentLayers = []
        groupRefLayer = self.getGroupForTheLayer(refLayer)
        for layer in self.layers :
            group = self.getGroupForTheLayer(layer)            
            if group == groupRefLayer : self.currentLayers.append(layer)              
        if groupRefLayer == None : groupRefLayer = refLayer    


    def getGroupForTheLayer(self, mapCanvasLayer):      
        root = QgsProject.instance().layerTreeRoot()
        groupLayer = None
        if not mapCanvasLayer : return None
        if not isinstance(mapCanvasLayer, QgsRasterLayer) : return None
        for layer in root.findLayers() :
            if isinstance(layer.layer(), QgsRasterLayer) :                
               if layer.layer().id() == mapCanvasLayer.id() :
                  if type(layer.parent()) == QgsLayerTreeGroup : groupLayer = layer.parent() ; break
        return groupLayer
          

    def zoomToLayerExtent(self, layer, target):
        try : 
            ds = gdal.Open(layer.source())
        
            minX, maxX = ds.GetGeoTransform()[0], (ds.RasterXSize*ds.GetGeoTransform()[1])+ds.GetGeoTransform()[0] 
            maxY, minY = ds.GetGeoTransform()[3], ds.GetGeoTransform()[3]+(ds.RasterYSize*ds.GetGeoTransform()[5])
            ds = None

            target.setExtent(QgsRectangle(minX, minY, maxX, maxY))
            target.refresh()
            
        except : return    
       


def actionsWindows(parent, name, title, width, height, isLegende):
        self = QMdiSubWindow(parent)
        self.setObjectName(name)
        self.setWindowTitle(title)

        if isLegende :
           self.setWindowFlags(Qt.WindowTitleHint | Qt.WindowStaysOnTopHint)
           self.setStyleSheet("QMdiSubWindow { border: 1px solid;    border-radius: 5px;   background-color: rgba(255, 255, 255, 210);}"
                              "QMdiSubWindow QWidget { background-color: transparent; border:1px;}"
                              "QMdiSubWindow:title { height: 18px; background-color: rgba(255, 255, 255, 210);}")           
        else :
           self.setWindowFlags(Qt.WindowTitleHint | Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint)
           self.setStyleSheet("QMdiSubWindow { border: 1px solid;    border-radius: 5px;   background-color: rgba(255, 255, 255, 210);}"
                              "QMdiSubWindow QWidget { background-color: transparent; border:1px;}"
                              "QMdiSubWindow:title { height: 18px; background-color: rgba(255, 255, 255, 210);}")
           self.setFixedSize(width, height)

        self.setGeometry(10, 10, width, height)
        
        self.oldPos = self.pos()

        return self

def ctrlMap(self, conteneur, complete, getPointsForExtent, simply, iX, iY, iW, iH, allActions):
        
         self.groupMap = QGroupBox(conteneur)

         self.actionsMapCanvas = QButtonGroup(conteneur) 
         self.actionsMapCanvas.setExclusive(True)

         if allActions :

             dicoCtrls = {"buttonCatchExtent" : ("catchextent.png", "Get extent for QGIS mapcanvas", 10, 20),
                          "buttonZoomIn" : ("zoomin.png", "Zoom in the map window", 40, 20),
                          "buttonZoomOut" : ("zoomout.png", "Zoom out the map window", 70, 20),
                          "buttonZoomAll" : ("zoomall.png", "Zoom all the map window", 100, 20),
                          "buttonZoomCurrentLayer" : ("zoomcurrentlayer.png", "Zoom to grid layer", 130, 20),
                          "buttonZoomPrevious" : ("zoomprev.png", "Previous zoom", 160, 20),
                          "buttonZoomNext" : ("zoomnext.png", "Next zoom", 190, 20),
                          "buttonSendExtent" : ("sendextent.png", "Send extent to Qgis mapcanvas", 220, 20),
                          "buttonPan" : ("pan.png", "Move the map window", 250, 20),
                          "buttonHotlink" : ("hotlink.png", "Load a raster file", 280, 20),
                          "buttonLoadExtent" : ("hotlinkextent.png", "Load the raster files for extent", 310, 20),
                          "buttonLoadSelected" : ("hotlinkselected.png", "Load the raster files for selected objects", 340, 20)
                          }
             

         else :
             dicoCtrls = {"buttonCatchExtent" : ("catchextent.png", "Get extent for QGIS mapcanvas", 10, 20),
                          "buttonZoomIn" : ("zoomin.png", "Zoom in the map window", 40, 20),
                          "buttonZoomOut" : ("zoomout.png", "Zoom out the map window", 70, 20),
                          "buttonZoomAll" : ("zoomall.png", "Zoom to all ...", 100, 20),
                          "buttonZoomPrevious" : ("zoomprev.png", "Previous zoom", 130, 20),
                          "buttonZoomNext" : ("zoomnext.png", "Next zoom", 160, 20),
                          "buttonSendExtent" : ("sendextent.png", "Send extent to Qgis mapcanvas", 190, 20),
                          "buttonPan" : ("pan.png", "Move the map window", 220, 20),
                          "boutonMagnify" : ("magnifycross.png", "Magnify particulat extent (CTRL+TAB to switch map)", 250, 20),
                          "buttonAddPermanentLayers" : ("add.png", "Add permanent vector layers", 280, 20),
                          "buttonDelPermanentLayers" : ("del.png", "Delete permanent vector layers", 310, 20)
                      }


         pos = 5

       
         for key in dicoCtrls :
      
                   icon = getThemeIcon(dicoCtrls[key][0])
                   toolTip = QApplication.translate("autoLoaderRasterPlugin", dicoCtrls[key][1])

                   button = myPushButton(self.groupMap)
                   
                   if key!= "buttonHotlink" : button.initPushButton(25, 25, pos, dicoCtrls[key][3], key, "", toolTip, True, icon, 25, 25, True)
                   else :  button.initPushButton(50, 25, pos, dicoCtrls[key][3], key, "", toolTip, True, icon, 50, 25, True)

                   if key in ("buttonZoomIn", "buttonZoomOut", "boutonMagnify", "buttonPan") : 

                      button.setStyleSheet(self.styleSheetPushButton)
                      button.setCheckable(True)

                      if key == "buttonZoomIn" : button.setChecked(True)
 
                      index = ("buttonZoomIn", "buttonZoomOut", "boutonMagnify", "buttonPan").index(key)
                      self.actionsMapCanvas.addButton(button, index)
                      

                   if key == "buttonHotlink" :
                      button.setStyleSheet(self.styleSheetPushButtonWithMenu)
                      button.setCheckable(True)

                      self.contextMnuToolsHotlink(button)
                   
                   setattr(self, key, button)
                   
                   if key == "buttonHotlink" : pos+= 50
                   else : pos+= 30 
                   

         sizeWidth = len(self.groupMap.children())*30
         self.groupMap.setGeometry(5, 10, sizeWidth+20, 50)           

         return sizeWidth



class toolMagnify(QgsMapTool):
    def __init__(self, parent, mapViewer, mapViewerBis, uiAlr):
        QgsMapTool.__init__(self, mapViewer)
        self.parent = parent
        self.imgMagnify = self.parent.imgMagnify
        self.labelCoordCursorPos = self.parent.labelCoordCursorPos
        self.delta = None
       
        self.mapViewer, self.mapViewerBis = mapViewer, mapViewerBis
        self.firstMapViewer = True
        self.status = 0
        self.cp = None
        self.cursor = QCursor(QPixmap(getThemeIcon("magnifycross.png")))
        self.timerMapTips = QTimer( self.mapViewer )

    def getDelta(self):
        delta = int(ceil(self.mapViewer.scale()) * 0.000000050) 
        return iff(delta < 10, 10, delta)

    def updateMapViewer(self) :
        if self.firstMapViewer : self.imgMagnify.setLayers(self.mapViewer.layers())        
        else : self.imgMagnify.setLayers(self.mapViewerBis.layers())
        self.imgMagnify.setDestinationCrs(self.imgMagnify.layers()[0].crs())

    def resetRb(self) :
        self.parent.rbDynaMire.reset()
        self.parent.rbDynaMire2.reset()
        self.parent.rb.reset()
        self.parent.rb2.reset()
        
    def updateRb(self):
        self.resetRb()
        extent = self.imgMagnify.extent()

        if self.firstMapViewer :

            if self.mapViewerBis.layers() == [] : return
            
            crs2 = self.mapViewerBis.layers()[0].crs()
            self.parent.rb.addGeometry(QgsGeometry().fromRect(extent))            
            extent2 = self.parent.transformer.transform(extent, QgsCoordinateTransform.ForwardTransform)  
            self.parent.rb2.addGeometry(QgsGeometry().fromRect(extent2), crs2)

            if self.cp :
                self.drawAxes(self.mapViewer.extent(), self.mapViewer.layers()[0].crs(), self.cp, self.parent.rbDynaMire)
                cp2 = self.parent.transformer.transform(self.cp, QgsCoordinateTransform.ForwardTransform) 
                self.drawAxes(self.mapViewerBis.extent(), crs2, cp2, self.parent.rbDynaMire2)
        else :
            
            crs2 = self.mapViewerBis.layers()[0].crs()            
            extent1 = self.parent.transformer.transform(extent, QgsCoordinateTransform.ReverseTransform)
            self.parent.rb.addGeometry(QgsGeometry().fromRect(extent1))
            self.parent.rb2.addGeometry(QgsGeometry().fromRect(extent), crs2)

            if self.cp :
                cp1 = self.parent.transformer.transform(self.cp, QgsCoordinateTransform.ReverseTransform)
                self.drawAxes(self.mapViewer.extent(), self.mapViewer.layers()[0].crs(), self.cp, self.parent.rbDynaMire)
                self.drawAxes(self.mapViewerBis.extent(), crs2, self.cp, self.parent.rbDynaMire2)

            

        
    def drawAxes(self, extent, crs, cp, rbDyna):
        if not cp : return
        if not extent : return
         
        point1, point2  = QgsPointXY(extent.xMinimum(), cp.y()), QgsPointXY(extent.xMaximum(), cp.y())
        geometry = QgsGeometry().fromPolylineXY([point1, point2])
        rbDyna.addGeometry(geometry, crs)

        point1, point2  = QgsPointXY(cp.x(), extent.yMaximum()), QgsPointXY(cp.x(), extent.yMinimum())
        geometry = QgsGeometry().fromPolylineXY([point1, point2])
        rbDyna.addGeometry(geometry, crs)        

    def zoomInMg(self): self.resetRb(); self.imgMagnify.zoomIn(); self.updateRb();
    def zoomOutMg(self): self.resetRb(); self.imgMagnify.zoomOut(); self.updateRb();

    def setVisibleCtrl(self, cond):
        self.imgMagnify.setVisible(cond)
        self.labelCoordCursorPos.setVisible(cond)
        
    def deactivate(self) :
        self.cp = None
        self.resetRb()
        self.setVisibleCtrl(False)
        self.firstMapViewer = True
           
    def activate(self) :
        self.updateMapViewer()
        self.setCursor(self.cursor)
        self.setVisibleCtrl(True)
        self.firstMapViewer = True

    def keyPressEvent(self, event) :
        if event.key() == Qt.Key_Tab :
           cond = (self.mapViewer.layers()[0].crs() == self.mapViewerBis.layers()[0].crs()) 
           if cond :
               self.firstMapViewer = not (self.firstMapViewer)
               self.updateMapViewer()
               self.updateRb()
           else : 
               title = QApplication.translate("autoLoaderRasterPlugin", "Information")
               msg = QApplication.translate("autoLoaderRasterPlugin","Switch impossible [projections différentes]!")
               self.parent.barInfo.pushMessage("%s :" % title, "%s" % (msg), Qgis.Critical, 3) #Warning
        elif event.key() == Qt.Key_Plus : self.zoomInMg()
        elif event.key() == Qt.Key_Minus : self.zoomOutMg()


    def canvasMoveEvent(self, e):
        #NEW 3.2.0
        point = QgsPointXY(e.pos().x(), e.pos().y())
        cond = (self.mapViewer.layers()[0].crs() != self.mapViewerBis.layers()[0].crs())
        if self.firstMapViewer : self.cp = self.toMapCoordinates(e.pos())
        else :
            self.cp = self.toMapCoordinates(self.mapViewer.layers()[0], point) if cond else  self.toMapCoordinates(e.pos())

        self.labelCoordCursorPos.setText("x : %s, y : %s" % (round(self.cp.x(), 4), round(self.cp.y(), 4)))
        self.resetRb()

        if not self.delta :        
            delta = self.getDelta()            
            minX, maxX = self.cp.x() - delta, self.cp.x() + delta
            minY, maxY = self.cp.y() - delta, self.cp.y() + delta
            extent = QgsRectangle(minX, minY, maxX, maxY)
            self.imgMagnify.setExtent(extent)
            self.delta = delta
        else : self.imgMagnify.setCenter(self.cp) ; extent = self.imgMagnify.extent()
        
        self.imgMagnify.refresh()

        self.updateRb()

        self.refExtent = extent if self.firstMapViewer else self.parent.transformer.transform(extent, QgsCoordinateTransform.ReverseTransform) 

        #POSITION / DIM MAPCANVAS        
        if e.pos().y()+345 < self.mapViewer.height() : self.imgMagnify.setGeometry(e.pos().x()+25, e.pos().y()+25, 320, 320)
        else : self.imgMagnify.setGeometry(e.pos().x()+25, e.pos().y()-305, 320, 320)
        self.labelCoordCursorPos.setGeometry(self.imgMagnify.x(), self.imgMagnify.y()-25, 320, 25)
        






